<?php

use ConnectpayPaymentGateway\Includes\CpConstants;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Plugin Name: ConnectPay Payment Gateway
 * Plugin URI: https://dev.connectpay.com/
 * Description: ConnectPay Payment Gateway plugin
 * Author: ConnectPay
 * Version: 1.2.1
 * Author URI:
 */

$wc_plugin_path = trailingslashit( WP_PLUGIN_DIR ) . 'woocommerce/woocommerce.php';

if ( in_array( $wc_plugin_path, wp_get_active_and_valid_plugins() ) ) {
	add_action( 'plugins_loaded', 'cp_payment_init', 11 );

	function cp_payment_init() {
		if ( class_exists( 'WC_Payment_Gateway' ) ) {
			require_once plugin_dir_path( __FILE__ ) . '/includes/CpConstants.php';
			require_once plugin_dir_path( __FILE__ ) . '/includes/class-wc-payment-gateway-cp.php';
			require_once plugin_dir_path( __FILE__ ) . '/includes/cp-checkout-description-fields.php';
			require_once plugin_dir_path( __FILE__ ) . '/includes/cp-api-methods.php';
			require_once plugin_dir_path( __FILE__ ) . '/includes/wp-cron/cp-task-get-payment-details-scheduler.php';

			load_plugin_textdomain( CpConstants::CP_TRANSLATIONS, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}
	}

	function add_cp_to_woo_payment_gateways( $gateways ) {
		$gateways[] = 'WC_CP_Payment_Gateway';

		return $gateways;
	}

	function permit_certificate_uploads( $existing_mimes ) {
		$existing_mimes['p8']  = 'text/plain';
		$existing_mimes['key'] = 'text/plain';
		$existing_mimes['p10'] = 'text/plain';
		$existing_mimes['csr'] = 'text/plain';
		$existing_mimes['cer'] = 'text/plain';
		$existing_mimes['crl'] = 'text/plain';
		$existing_mimes['p7c'] = 'text/plain';
		$existing_mimes['crt'] = 'text/plain';
		$existing_mimes['der'] = 'text/plain';
		$existing_mimes['crl'] = 'text/plain';
		$existing_mimes['pem'] = 'text/plain';
		$existing_mimes['p12'] = 'text/plain';
		$existing_mimes['pfx'] = 'text/plain';
		$existing_mimes['p7b'] = 'text/plain';
		$existing_mimes['spc'] = 'text/plain';
		$existing_mimes['p7r'] = 'text/plain';

		return $existing_mimes;
	}

	function add_checkout_translations_to_db() {
		$terms_of_use_lt_url      = 'https://connectpay.com/Mokejimu-internetu-priemimo-naudojimo-salygos.pdf';
		$terms_of_use_other_url   = 'https://connectpay.com/Online-Payment-Acceptance-Terms-of-Use.pdf';
		$privacy_policy_lt_url    = 'https://connectpay.com/lt/privatumo-politika/';
		$privacy_policy_other_url = 'https://connectpay.com/privacy-policy/';

		$translations = [
			'description'      => [
				'en_US' => [
					'main_part'          => 'Service is provided by ConnectPay UAB. By using our services, you agree to our',
					'terms_of_use'       => 'Terms of Use',
					'and'                => 'and',
					'privacy_policy'     => 'Privacy Policy',
					'terms_of_use_url'   => $terms_of_use_other_url,
					'privacy_policy_url' => $privacy_policy_other_url,
				],
				'de_DE' => [
					'main_part'          => 'Zahlungsdienstleister UAB ConnectPay. Indem Sie unsere Leistungen nutzen, akzeptieren Sie unsere',
					'terms_of_use'       => 'AGB',
					'and'                => 'und',
					'privacy_policy'     => 'Datenschutzrichtlinie',
					'terms_of_use_url'   => $terms_of_use_other_url,
					'privacy_policy_url' => $privacy_policy_other_url,
				],
				'fi'    => [
					'main_part'          => 'Maksusi välittää ConnectPay UAB. Palveluamme käyttämällä hyväksyt meidän',
					'terms_of_use'       => 'käyttöoehtomme',
					'and'                => 'ja',
					'privacy_policy'     => 'tietosuojakäytäntömme',
					'terms_of_use_url'   => $terms_of_use_other_url,
					'privacy_policy_url' => $privacy_policy_other_url,
				],
				'nl_NL' => [
					'main_part'          => 'Betaalinitiatie wordt verstrekt door ConnectPay UAB. Door gebruik te maken van onze diensten, stemt u in met onze',
					'terms_of_use'       => 'gebruiksvoorwaarden',
					'and'                => 'en',
					'privacy_policy'     => 'privacybeleid',
					'terms_of_use_url'   => $terms_of_use_other_url,
					'privacy_policy_url' => $privacy_policy_other_url,
				],
				'sv_SE' => [
					'main_part'          => 'Betalningsinitiering levereras av ConnectPay UAB: Genom att använda våra tjänster accepterar du våra',
					'terms_of_use'       => 'Användningsvillkor',
					'and'                => 'och',
					'privacy_policy'     => 'Integritetspolicy',
					'terms_of_use_url'   => $terms_of_use_other_url,
					'privacy_policy_url' => $privacy_policy_other_url,
				],
				'lt_LT' => [
					'main_part'          => 'Paslaugą teikia UAB "ConnectPay". Naudodamiesi mūsų paslaugomis Jūs sutinkate su',
					'terms_of_use'       => 'Naudojimosi sąlygomis',
					'and'                => 'ir',
					'privacy_policy'     => 'Privatumo politika',
					'terms_of_use_url'   => $terms_of_use_lt_url,
					'privacy_policy_url' => $privacy_policy_lt_url,
				],
			],
			'by_card'          => [
				'en_US' => [
					'main_part' => 'By card',
				],
				'de_DE' => [
					'main_part' => 'Per Karte',
				],
				'fi'    => [
					'main_part' => 'Kortilla',
				],
				'nl_NL' => [
					'main_part' => 'Per kaart',
				],
				'sv_SE' => [
					'main_part' => 'Med kort',
				],
				'lt_LT' => [
					'main_part' => 'Mokėjimo kortele',
				],
			],
			'by_bank_transfer' => [
				'en_US' => [
					'main_part' => 'By bank transfer',
				],
				'de_DE' => [
					'main_part' => 'Per Banküberweisung',
				],
				'fi'    => [
					'main_part' => 'Pankkisiirrolla',
				],
				'nl_NL' => [
					'main_part' => 'Via bank overdracht',
				],
				'sv_SE' => [
					'main_part' => 'Via banköverföring',
				],
				'lt_LT' => [
					'main_part' => 'Banko pavedimu',
				],
			],
		];

		update_option( 'cp_checkout_translations', $translations );
	}

	// Notifications API endpoint functionality
	function notifications_route() {
		register_rest_route( 'cp', 'notifications', [
			'methods'             => 'POST',
			'callback'            => 'order_notifications',
			'permission_callback' => '__return_true'
		] );
	}

	function callback_route() {
		register_rest_route( 'cp', 'callback', [
			'methods'             => 'GET',
			'callback'            => 'check_callback_redirection',
			'permission_callback' => '__return_true'
		] );
	}

	function get_updated_wc_order_status( $order_status ) {
		$cp_pending_order_statuses    = [
			'initiated'       => 'initiated',
			'pendingMerchant' => 'pendingmerchant',
			'inProgress'      => 'inprogress',
			'processing'      => 'processing',
			'pendingProvider' => 'pendingprovider',
		];
		$cp_processing_order_statuses = [
			'approved' => 'approved',
			'settled'  => 'settled',
		];
		$cp_failed_order_statuses     = [
			'failed'    => 'failed',
			'abandoned' => 'abandoned'
		];

		if ( in_array( $order_status, $cp_pending_order_statuses ) ) {
			$new_wc_order_status = 'wc-pending';
		} else if ( in_array( $order_status, $cp_processing_order_statuses ) ) {
			$new_wc_order_status = 'wc-processing';
		} else if ( in_array( $order_status, $cp_failed_order_statuses ) ) {
			$new_wc_order_status = 'wc-failed';
		} else {
			$new_wc_order_status = false;
		}

		return $new_wc_order_status;
	}

	function order_notifications( $data ) {
		$delay_in_seconds  = 5;
		$cp_payment_method = 'cp_payment_gateway';
		$order_status      = strtolower( $data['status']['code'] );
		$first_name        = $data['consumer']['firstName'];
		$last_name         = $data['consumer']['lastName'];
		$email             = $data['consumer']['email'];
		$total             = $data['payment']['instructedAmount']['amount'];
		$final_statuses    = [
			'initiated',
			'pendingMerchant',
			'inProgress',
		];

		if ( in_array( $order_status, $final_statuses, true ) ) {
			return [
				'success' => 'Order is already with the final status'
			];
		}

		// remittanceInformationUnstructured = Order ID
		$order = wc_get_order( $data['payment']['remittanceInformationUnstructured'] );

		if ( false === $order ) {
			$cp_date_created           = new DateTime( $data['status']['createdAt'] );
			$cp_date_created_threshold = new DateTime( $data['status']['createdAt'] );

			$cp_date_created_threshold->modify( '-' . $delay_in_seconds . ' seconds' );
			$from = $cp_date_created_threshold->getTimestamp();
			$to   = $cp_date_created->getTimestamp();

			$query = new WC_Order_Query( [
				'limit'              => - 1,
				'orderby'            => 'date',
				'date_created'       => $from . '...' . $to,
				'billing_first_name' => $first_name,
				'billing_last_name'  => $last_name,
				'billing_email'      => $email,
				'total'              => $total,
				'payment_method'     => $cp_payment_method,
				'order'              => 'DESC',
				'return'             => 'objects',
			] );

			$orders = $query->get_orders();

			if ( empty( $orders ) ) {
				return [
					'error' => 'No such order'
				];
			}

			if ( count( $orders ) > 1 ) {
				return [
					'error' => 'Multiple orders returned'
				];
			}

			$order = $orders[0];
		}

		$transient_key = 'cp_notification_lock_' . $order->get_id();
		set_transient( $transient_key, true, $delay_in_seconds );

		// Make sure we got the latest data.
		$payment_id          = $order->get_meta( 'cp_payment_id' );
		$newest_data         = get_payment_details_method( $payment_id );
		$is_terminal         = $newest_data['status']['isTerminal'];
		$new_wc_order_status = get_updated_wc_order_status( strtolower( $newest_data['status']['code'] ) );

		if ( $new_wc_order_status === get_updated_wc_order_status( $order->get_status() ) ) {
			delete_transient( $transient_key );
			return [
				'success' => 'Order is with the same status'
			];
		}

		if ( $is_terminal ) {
			$is_terminal_meta = $order->get_meta( 'cp_is_terminal' );

			if ( 'false' === $is_terminal_meta ) {
				update_post_meta( $order->get_id(), 'cp_is_terminal', 'true' );
			} else if ( 'true' === $is_terminal_meta ) {
				delete_transient( $transient_key );
				return [
					'warning' => 'Order payment status is already terminal'
				];
			}
		}

		if ( false === $new_wc_order_status ) {
			delete_transient( $transient_key );
			return [
				'error' => 'Something went wrong'
			];
		}

		if ( $is_terminal && 'wc-processing' === $new_wc_order_status ) {
			$order->payment_complete();
			delete_transient( $transient_key );

			return [
				'success' => 'Order payment status updated (processing - terminal)'
			];
		} else if ( $is_terminal && 'wc-failed' === $new_wc_order_status ) {
			wc_increase_stock_levels( $order->get_id() );
			$order->update_status( 'wc-failed' );
			delete_transient( $transient_key );

			return [
				'success' => 'Order payment status updated (failed - terminal)'
			];
		}

		delete_transient( $transient_key );

		return [
			'success' => 'Order status is pending. No need to update.',
		];
	}

	function check_callback_redirection( $data ) {
		$error_cookie_name = 'order_error_message';

		if ( isset( $_GET['order'] ) && ! empty( $_GET['order'] ) ) {
			$order_id = sanitize_text_field( $_GET['order'] );
			$order    = wc_get_order( $order_id );

			if ( false === $order ) {
				setcookie( $error_cookie_name, __( 'No such order', CpConstants::CP_TRANSLATIONS ), time() + 2, '/' );
				wp_redirect( wc_get_cart_url() );
				exit;
			}

			$payment_id = $order->get_meta( 'cp_payment_id' );

			$response            = get_payment_details_method( $payment_id );
			$is_error            = isset( $response['error'] ) && ! empty( $response['error'] );
			$processing_statuses = [ 'Approved', 'Settled' ];

			// Additionally, check if we get terminal already
			if (
				$response['status']['isTerminal'] &&
				in_array( $response['status']['code'], $processing_statuses, true ) &&
				! $order->get_meta( 'cp_is_terminal' )
			) {
				update_post_meta( $order_id, 'cp_is_terminal', 'true' );
			}

			$new_wc_order_status   = get_updated_wc_order_status( strtolower($response['status']['code']) );
			$local_wc_order_status = get_updated_wc_order_status( $order->get_status() );

			if ( false === $new_wc_order_status ) {
				return [
					'error' => 'Something went wrong',
				];
			}

			$transient_key = 'cp_notification_lock_' . $order->get_id();

			if ( $local_wc_order_status !== $new_wc_order_status && ! get_transient( $transient_key ) ) {
				$order->update_status( $new_wc_order_status );
			}

			if ( $is_error ) {
				setcookie( $error_cookie_name, __( $response['error'], CpConstants::CP_TRANSLATIONS ), time() + 2, '/' );
				wp_redirect( wc_get_cart_url() );
				exit;
			} else {
				$payment_status  = strtolower( $response['status']['code'] );
				$failed_statuses = [ 'failed', 'abandoned' ];

				if ( in_array( $payment_status, $failed_statuses ) ) {
					setcookie( $error_cookie_name, __( 'Payment failed', CpConstants::CP_TRANSLATIONS ), time() + 2, '/' );
					wp_redirect( wc_get_cart_url() );
					exit;
				} else {
					WC()->cart->empty_cart();
					wc_reduce_stock_levels( $order_id );
					wp_redirect( $order->get_meta( 'cp_order_success_url' ) );
					exit;
				}
			}
		} else {
			setcookie( $error_cookie_name, __( 'Error receiving status', CpConstants::CP_TRANSLATIONS ), time() + 2, '/' );
			wp_redirect( wc_get_cart_url() );
			exit;
		}
	}

	function show_payment_failed_error_message() {
		if ( isset( $_COOKIE['order_error_message'] ) ) {
			wc_add_notice( sanitize_text_field( $_COOKIE['order_error_message'] ), 'error' );
		}
	}

	function cp_admin_enqueue_scripts( $hook ) {
		global $post;
		if ( ( $hook === 'post.php' ) && 'shop_order' === $post->post_type ) {
			wp_enqueue_script( 'cp_scripts', plugin_dir_url( __FILE__ ) . 'assets/js/cp-admin-front-scripts.js', [ 'jquery' ], '1.0' );
		}
	}

	function cp_enqueue_scripts() {
		wp_enqueue_script( 'my-custom-script', plugins_url( 'assets/js/cp-front-scripts.js', __FILE__ ), [ 'jquery' ], '1.0', true );
	}

	add_filter( 'woocommerce_payment_gateways', 'add_cp_to_woo_payment_gateways' );

	add_filter( 'upload_mimes', 'permit_certificate_uploads' );

	add_action( 'rest_api_init', 'notifications_route' );

	add_action( 'rest_api_init', 'callback_route' );

	add_action( 'init', 'add_checkout_translations_to_db' );

	add_action( 'woocommerce_check_cart_items', 'show_payment_failed_error_message' );

	add_action( 'admin_enqueue_scripts', 'cp_admin_enqueue_scripts' );

	add_action( 'wp_enqueue_scripts', 'cp_enqueue_scripts' );

	require_once plugin_dir_path( __FILE__ ) . '/includes/wp-cron/cp-task-get-payment-details-scheduler.php';
} else {
	return;
}
