<?php

add_filter('cp_get_providers', 'cp_get_providers_method');
add_filter('cp_get_payment_details', 'cp_get_payment_details_method', 1, 2);
add_filter('cp_initiate_payment', 'cp_initiate_payment_method', 1, 2);

function cp_get_providers_method( $payment_providers ) {
	$country_code = get_option('cp_payment_country');
	$payment_providers = [];

	if ('DE' !== $country_code) {
		$request_url = 'https://api2-stage.connectpay.com/merchant/providers';
		$req_method = 'get';
		$page_no = '1';
		$page_size = '10000';

		$request_url = add_query_arg([
			'countryCode' => $country_code,
			'pageNo' => $page_no,
			'pageSize' => $page_size
		], $request_url);

		$request_url_for_signature = strtolower($request_url);
		$req_method = strtolower($req_method);
		$req_body = strtolower('');

		// Get URL without https:// part
		$request_url_for_signature = explode('//', $request_url_for_signature)[1];

		// Data to sign signature
		$data_to_sign = $req_method . '|' . $request_url_for_signature . '|' . $req_body;

		// Signature creation part
		$merchant_api_signature = get_signature($data_to_sign);

		$wp_request_headers = [
			'Authorization' => get_authorization_token(),
			'X-Request-ID' => get_guid(),
			'Accept' => 'application/json;version=2',
			'X-Signature' => $merchant_api_signature
		];

		$response = wp_remote_request(
			$request_url,
			[
				'method'  => 'GET',
				'headers' => $wp_request_headers
			]
		);

		if (wp_remote_retrieve_response_code($response) === 200) {
			$payment_providers = json_decode(wp_remote_retrieve_body($response), true)['providers'];
		} else {
			$error = wp_remote_retrieve_response_code($response) . ' ' . wp_remote_retrieve_response_message($response);
			set_request_error_cookie($error);
			$payment_providers = [
				'error' => $error
			];
		}
	}

	return $payment_providers;
}

function cp_get_payment_details_method( $payment_details, $payment_id ) {
	$payment_details = [];

	$request_url = 'https://api2-stage.connectpay.com/merchant/payments/' . $payment_id;
	$req_method = 'get';

	$request_url_for_signature = strtolower($request_url);
	$req_method = strtolower($req_method);
	$req_body = strtolower('');

	// Get URL without https:// part
	$request_url_for_signature = explode('//', $request_url_for_signature)[1];

	// Data to sign signature
	$data_to_sign = $req_method . '|' . $request_url_for_signature . '|' . $req_body;

	// Signature creation part
	$merchant_api_signature = get_signature($data_to_sign);

	$wp_request_headers = [
		'Authorization' => get_authorization_token(),
		'X-Request-ID' => get_guid(),
		'Accept' => 'application/json;version=2',
		'X-Signature' => $merchant_api_signature
	];

	$response = wp_remote_request(
		$request_url,
		[
			'method'  => 'GET',
			'headers' => $wp_request_headers
		]
	);

	if (wp_remote_retrieve_response_code($response) === 200) {
		$payment_details = json_decode(wp_remote_retrieve_body($response), true);
	} else {
		$error = wp_remote_retrieve_response_code($response) . ' ' . wp_remote_retrieve_response_message($response) . ' (' . $response['body'] . ')';
		set_request_error_cookie($error);
		$payment_details = [
			'error' => $error
		];
	}

	return $payment_details;
}

function cp_initiate_payment_method( $payment_data, $body ) {
	$provider_country_code = get_option('cp_payment_country');
	$user_language = strtoupper(explode('_', get_user_locale())[0]);
	$request_url = 'https://api2-stage.connectpay.com/merchant/payments';
	$brand_id = get_option('woocommerce_cp_payment_gateway_settings')['cp_brand_id'];
	$notification_url = get_home_url() . '/?rest_route=/cp/notifications';
	$ip = $body['ip'];

//	Uncomment for local testing, CP API doesn't support local IP address
//	$ip = '12.12.12.12';

	$req_method = 'post';
	$request_url_for_signature = strtolower($request_url);
	$req_method = strtolower($req_method);

	// Get URL without https:// part
	$request_url_for_signature = explode('//', $request_url_for_signature)[1];

	// Request body
	$request_body = [
		'merchant' => [
			'brandId' => $brand_id,
			'redirectUrl' => $body['callback'],
			'notificationUrl' => $notification_url,
			'language' => $user_language,
		],
		'payment' => [
			'paymentMethod' => $body['method'],
			'providerCountryCode' => $provider_country_code,
			'instructedAmount' => [
				'amount' => number_format((float)$body['total'], 2, '.', ''),
				'currency' => $body['currency'],
			],
			'remittanceInformationUnstructured' => $body['orderId']
		],
		'identifiers' => [
			'merchantReference' => get_guid(true),
			'endToEndId' => random_string(30),
		],
		'consumer' => [
			'firstName' => $body['firstName'],
			'lastName' => $body['lastName'],
			'ipAddress' => $ip,
			'consumerId' => get_guid(true),
			'email' => $body['email'],
		]
	];

	if (isset($body['providerId']) && !empty($body['providerId'])) {
		$request_body['payment']['providerId'] = $body['providerId'];
	}

	if (isset($body['phone']) && !empty($body['phone'])) {
		$request_body['consumer']['phoneNo'] = $body['phone'];
	}

	$request_body_json = json_encode($request_body);
	$request_body_json = preg_replace('/\s+/', '', $request_body_json);

	// Data to sign signature
	$data_to_sign = $req_method . '|' . $request_url_for_signature . '|' . $request_body_json;

	// Signature creation part
	$merchant_api_signature = get_signature($data_to_sign);

	// Request headers
	$wp_request_headers = [
		'Authorization' => get_authorization_token(),
		'X-Request-ID' => get_guid(),
		'Accept' => 'application/json;version=2',
		'Content-Type' => 'application/json',
		'X-Signature' => $merchant_api_signature,
		'Connection' => 'keep-alive',
	];

	// Response from POST Initiate Payment request
	$response = wp_remote_post(
		$request_url,
		[
			'method'  => 'POST',
			'headers' => $wp_request_headers,
			'body' => $request_body_json
		]
	);

	$status_code = wp_remote_retrieve_response_code($response);

	if (200 === $status_code || 201 === $status_code ) {
		$payment_data = json_decode(wp_remote_retrieve_body($response), true);
	} else {
		$error = wp_remote_retrieve_response_code($response) . ' ' . wp_remote_retrieve_response_message($response) . ' (' . $response['body'] . ')';
		set_request_error_cookie($error);
		$payment_data = [
			'error' => $error
		];
	}

	return $payment_data;
}

function get_signature( $data_to_sign ) {
	$private_key = base64_decode(get_option('cp_private_key'));
	$signature = null;
	$alg = 'sha256WithRSAEncryption';
	openssl_sign($data_to_sign, $signature, $private_key, $alg);

	return base64_encode($signature);
}

function get_authorization_token() {
	$client_id = get_option('woocommerce_cp_payment_gateway_settings')['cp_client_id'];
	$client_secret = get_option('woocommerce_cp_payment_gateway_settings')['cp_client_secret'];

	$token = 'Basic ' . base64_encode($client_id . ':' . $client_secret);

	return $token;
}

function set_request_error_cookie( $error ) {
	$cookie_name = 'request_error';
	$cookie_value = $error;
	setcookie($cookie_name, $cookie_value, time() + 1, '/'); // 86400 = 1 day
}

function get_guid( $isMerchantReference = false ) {
	if (function_exists('com_create_guid')) {
		return com_create_guid();
	} else {
		mt_srand((float) microtime()*10000);
//		$charid = strtoupper(md5(uniqid(rand(), true)));
		$charid = md5(uniqid(rand(), true));
		$hyphen = chr(45);// "-"
		if ($isMerchantReference) {
			$uuid = substr($charid, 0, 8) . $hyphen
				. substr($charid, 8, 4) . $hyphen
				. substr($charid, 12, 4) . $hyphen
				. substr($charid, 16, 4) . $hyphen
				. substr($charid, 20, 12);
		} else {
			$uuid = chr(123)// "{"
				. substr($charid, 0, 8) . $hyphen
				. substr($charid, 8, 4) . $hyphen
				. substr($charid, 12, 4) . $hyphen
				. substr($charid, 16, 4) . $hyphen
				. substr($charid, 20, 12)
				. chr(125);// "}"
		}
		return $uuid;
	}
}

function random_string( $length ) {
	// String of all alphanumeric character
	$str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

	// Shuffle the $str_result and returns substring of specified length
	return substr(str_shuffle($str_result), 0, $length);
}
